package at.ac.uibk.gitsearch.service;

import at.ac.uibk.gitsearch.domain.UserWatchList;
import at.ac.uibk.gitsearch.repository.UserWatchListRepository;
import at.ac.uibk.gitsearch.repository.search.UserWatchListSearchRepository;
import at.ac.uibk.gitsearch.service.dto.UserWatchListDTO;
import at.ac.uibk.gitsearch.service.mapper.UserWatchListMapper;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import org.springframework.data.domain.Page;
import org.springframework.data.domain.Pageable;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import java.util.List;
import java.util.Optional;
import java.util.stream.Collectors;

import static org.elasticsearch.index.query.QueryBuilders.*;

/**
 * Service Implementation for managing {@link UserWatchList}.
 */
@Service
@Transactional
public class UserWatchListService {

    private final Logger log = LoggerFactory.getLogger(UserWatchListService.class);

    private final UserWatchListRepository userWatchListRepository;

    private final UserWatchListMapper userWatchListMapper;

    private final UserWatchListSearchRepository userWatchListSearchRepository;

    public UserWatchListService(UserWatchListRepository userWatchListRepository, UserWatchListMapper userWatchListMapper, UserWatchListSearchRepository userWatchListSearchRepository) {
        this.userWatchListRepository = userWatchListRepository;
        this.userWatchListMapper = userWatchListMapper;
        this.userWatchListSearchRepository = userWatchListSearchRepository;
    }

    /**
     * Save a userWatchList.
     *
     * @param userWatchListDTO the entity to save.
     * @return the persisted entity.
     */
    public UserWatchListDTO save(UserWatchListDTO userWatchListDTO) {
        log.debug("Request to save UserWatchList : {}", userWatchListDTO);
        UserWatchList userWatchList = userWatchListMapper.toEntity(userWatchListDTO);
        userWatchList = userWatchListRepository.save(userWatchList);
        UserWatchListDTO result = userWatchListMapper.toDto(userWatchList);
        userWatchListSearchRepository.save(userWatchList);
        return result;
    }

    /**
     * Get all the userWatchLists.
     *
     * @param pageable the pagination information.
     * @return the list of entities.
     */
    @Transactional(readOnly = true)
    public Page<UserWatchListDTO> findAll(Pageable pageable) {
        log.debug("Request to get all UserWatchLists");
        return userWatchListRepository.findAll(pageable)
            .map(userWatchListMapper::toDto);
    }

    /**
     * Get all the userWatchLists.
     *
     * @param pageable the pagination information.
     * @return the list of entities.
     */
    @Transactional(readOnly = true)
    public List<UserWatchListDTO> findByUserIdIsCurrentUser() {
        log.debug("Request to get all UserWatchLists");
        return userWatchListRepository.findByUserIdIsCurrentUser().stream()
            .map(userWatchListMapper::toDto).collect(Collectors.toList());
    }


    /**
     * Get one userWatchList by id.
     *
     * @param id the id of the entity.
     * @return the entity.
     */
    @Transactional(readOnly = true)
    public Optional<UserWatchListDTO> findOne(Long id) {
        log.debug("Request to get UserWatchList : {}", id);
        return userWatchListRepository.findById(id)
            .map(userWatchListMapper::toDto);
    }

    /**
     * Delete the userWatchList by id.
     *
     * @param id the id of the entity.
     */
    public void delete(Long id) {
        log.debug("Request to delete UserWatchList : {}", id);
        userWatchListRepository.deleteById(id);
        userWatchListSearchRepository.deleteById(id);
    }

    /**
     * Search for the userWatchList corresponding to the query.
     *
     * @param query the query of the search.
     * @param pageable the pagination information.
     * @return the list of entities.
     */
    @Transactional(readOnly = true)
    public Page<UserWatchListDTO> search(String query, Pageable pageable) {
        log.debug("Request to search for a page of UserWatchLists for query {}", query);
        return userWatchListSearchRepository.search(queryStringQuery(query), pageable)
            .map(userWatchListMapper::toDto);
    }
}
