package at.ac.uibk.gitsearch.service;

import at.ac.uibk.gitsearch.domain.WatchListEntry;
import at.ac.uibk.gitsearch.repository.WatchListEntryRepository;
import at.ac.uibk.gitsearch.repository.search.WatchListEntrySearchRepository;
import at.ac.uibk.gitsearch.service.dto.WatchListEntryDTO;
import at.ac.uibk.gitsearch.service.mapper.WatchListEntryMapper;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import org.springframework.data.domain.Page;
import org.springframework.data.domain.Pageable;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import java.util.Optional;

import static org.elasticsearch.index.query.QueryBuilders.*;

/**
 * Service Implementation for managing {@link WatchListEntry}.
 */
@Service
@Transactional
public class WatchListEntryService {

    private final Logger log = LoggerFactory.getLogger(WatchListEntryService.class);

    private final WatchListEntryRepository watchListEntryRepository;

    private final WatchListEntryMapper watchListEntryMapper;

    private final WatchListEntrySearchRepository watchListEntrySearchRepository;

    public WatchListEntryService(WatchListEntryRepository watchListEntryRepository, WatchListEntryMapper watchListEntryMapper, WatchListEntrySearchRepository watchListEntrySearchRepository) {
        this.watchListEntryRepository = watchListEntryRepository;
        this.watchListEntryMapper = watchListEntryMapper;
        this.watchListEntrySearchRepository = watchListEntrySearchRepository;
    }

    /**
     * Save a watchListEntry.
     *
     * @param watchListEntryDTO the entity to save.
     * @return the persisted entity.
     */
    public WatchListEntryDTO save(WatchListEntryDTO watchListEntryDTO) {
        log.debug("Request to save WatchListEntry : {}", watchListEntryDTO);
        WatchListEntry watchListEntry = watchListEntryMapper.toEntity(watchListEntryDTO);
        watchListEntry = watchListEntryRepository.save(watchListEntry);
        WatchListEntryDTO result = watchListEntryMapper.toDto(watchListEntry);
        watchListEntrySearchRepository.save(watchListEntry);
        return result;
    }

    /**
     * Get all the watchListEntries.
     *
     * @param pageable the pagination information.
     * @return the list of entities.
     */
    @Transactional(readOnly = true)
    public Page<WatchListEntryDTO> findAll(Pageable pageable) {
        log.debug("Request to get all WatchListEntries");
        return watchListEntryRepository.findAll(pageable)
            .map(watchListEntryMapper::toDto);
    }


    /**
     * Get one watchListEntry by id.
     *
     * @param id the id of the entity.
     * @return the entity.
     */
    @Transactional(readOnly = true)
    public Optional<WatchListEntryDTO> findOne(Long id) {
        log.debug("Request to get WatchListEntry : {}", id);
        return watchListEntryRepository.findById(id)
            .map(watchListEntryMapper::toDto);
    }

    /**
     * Delete the watchListEntry by id.
     *
     * @param id the id of the entity.
     */
    public void delete(Long id) {
        log.debug("Request to delete WatchListEntry : {}", id);
        watchListEntryRepository.deleteById(id);
        watchListEntrySearchRepository.deleteById(id);
    }

    /**
     * Search for the watchListEntry corresponding to the query.
     *
     * @param query the query of the search.
     * @param pageable the pagination information.
     * @return the list of entities.
     */
    @Transactional(readOnly = true)
    public Page<WatchListEntryDTO> search(String query, Pageable pageable) {
        log.debug("Request to search for a page of WatchListEntries for query {}", query);
        return watchListEntrySearchRepository.search(queryStringQuery(query), pageable)
            .map(watchListEntryMapper::toDto);
    }
}
